#! /usr/bin/ruby1.9.1
# -*- coding: utf-8 -*-

ADAPTER = 1+8                   # adapterN
ADAPTER2 = 3+8                  # adapterN
CH = 13                         # 地上波のCH
CH2 = 14                        # 地上波のCH

# ◆TEST1
# 807の件(単にdemuxから読めなくなる)
# しばらく読めるが、少しすると読めなくなる(Psisi〜の出力が止まる)
# (filter_loopのpoolがPOLERRを返すため)
# 生のdemux(ADAPTER = 1)では問題なし
TEST = 1

# ◆TEST2
# 772の件(dvr+demuxでdvr側ロック)
# dvr側がロックのパターンが再現しない＼(^o^)／ｵﾜﾀ
# 元々1回こっきりしか起こせませんでしたが、
# 当時問題の起きた別プログラムでも再現できません
# （別プログラムはチューニング+SDTでこっちはEITですが、まぁ同じだろうし...）
# demux側が読めないパターンは807と同問題でした
#TEST = 2

# ◆TEST3
# 838(≒743?)の件(dvr+dvr)
# 838と同じような動作を狙ってみました
# うちでは問題ありませんでした
#TEST = 3

############################################################

require './v4ldvb/v4ldvb'
require './section'

def get_pat(fe)
  DvbDemux.open(fe) do |demux|
    demux.setsctfilter(PID::PAT, Table::PAT::TAG)
    t = Table.timeout(fe.type, Table::PAT::TAG)
    demux.section(t) do |psisi|
      table = Table.analyze(psisi)
      if table.class == Table::PAT
        return table
      end
    end
  end
  nil
end

def get_pmt(fe, pid)
  DvbDemux.open(fe) do |demux|
    demux.setsctfilter(pid, Table::PMT::TAG)
    t = Table.timeout(fe.type, Table::PMT::TAG)
    demux.section(t) do |psisi|
      table = Table.analyze(psisi)
      if table.class == Table::PMT
        return table
      end
    end
  end
  nil
end

def analyze_pmt(pmt)
  filter = []
  pmt.data.each do |data|
    case data.stream_type
    when 0x02 # MPEG2 video
      data.descriptor.each do |desc|
        if desc.class == Descriptor::StreamIdentifier
          if desc.component_tag == 0x00
            filter << [:video, data.elementary_pid]
            break
          end
        end
      end
    when 0x0f # MPEG2 AAC
      data.descriptor.each do |desc|
        if desc.class == Descriptor::StreamIdentifier
          if desc.component_tag == 0x10 ||
              desc.component_tag == 0x85
            filter << [:audio, data.elementary_pid]
            break
          end
        end
      end
    when 0x06 # 字幕
      data.descriptor.each do |desc|
        if desc.class == Descriptor::StreamIdentifier
          if desc.component_tag == 0x30
            filter << [:subtitle, data.elementary_pid]
            break
          end
        end
      end
    when 0x0d # データカルーセル
    end
  end
  filter
end

# PID指定でdvr読み出し
def main(fe, s, filter)
  DvbDemux.open(fe, filter.size) do |demuxs|
    demuxs.each_with_index do |demux, ind|
      pestype, pid = *filter[ind]
      demux.setpesfilter(pid, pestype)
    end
    DvbDvr.open(fe) do |dvr|
      dvr.setfilter([PID::EIT], false)
      dvr.section(3000, s) do |psisi_pes|
        if psisi_pes.class == Pes
          if psisi_pes.hdr
            if psisi_pes.stream_id == 0xc0
            end
          else
          end
        else
          #p psisi_pes
          #table = Table.analyze(psisi_pes)
          #p table
          p fe
        end
      end
    end
  end
end

# PID指定なしでdvr読み出し
def main2(fe, s)
  DvbDemux.open(fe) do |demux|
    demux.setpesfilter
    DvbDvr.open(fe) do |dvr|
      dvr.setfilter([PID::EIT], false)
      dvr.section(3000, s) do |psisi_pes|
        if psisi_pes.class == Pes
          if psisi_pes.hdr
            if psisi_pes.stream_id == 0xc0
            end
          else
          end
        else
          #p psisi_pes
          #table = Table.analyze(psisi_pes)
          #p table
          p fe
        end
      end
    end
  end
end

# demux読み出し
def sub(fe)
  DvbDemux.open(fe) do |demux|
    demux.setsctfilter(PID::EIT)
    demux.section(3000) do |psisi|
      p psisi
      #table = Table.analyze(psisi)
      #p table
    end
  end
end

case TEST
when 1
  adapter = DvbAdapter.new(ADAPTER, 0, :te)
  DvbFrontend.open(adapter) do |fe|
    fe.tune(CH)
    sub(fe)
  end

when 2
  th_sub = Thread.start do
    sleep 1
    adapter = DvbAdapter.new(ADAPTER2, 0, :te)
    DvbFrontend.open(adapter) do |fe|
      fe.tune(CH2)
      sub(fe)
    end
  end
  th_sub.abort_on_exception = true

  adapter = DvbAdapter.new(ADAPTER, 0, :te)
  DvbFrontend.open(adapter) do |fe|
    fe.tune(CH)

    filter = []
    pat = get_pat(fe)
    break unless pat
    pid = nil
    ind = pat.data.index { |data| data.program_number != 0 }
    pid = pat.data[ind].pid if ind
    break unless pid
    pmt = get_pmt(fe, pid)
    filter = analyze_pmt(pmt)
    filter << [:other, PID::PAT]
    filter << [:other, pid]
    filter << [:other, PID::EIT]
    p filter

    File.open("/dev/null", "w") do |s|
      main(fe, s, filter)
    end
  end

when 3
  th_sub = Thread.start do
    sleep 1
    adapter = DvbAdapter.new(ADAPTER2, 0, :te)
    DvbFrontend.open(adapter) do |fe|
      fe.tune(CH)

      File.open("/dev/null", "w") do |s|
        main2(fe, s)
      end
    end
  end
  th_sub.abort_on_exception = true

  adapter = DvbAdapter.new(ADAPTER, 0, :te)
  DvbFrontend.open(adapter) do |fe|
    fe.tune(CH)

    filter = []
    pat = get_pat(fe)
    break unless pat
    pid = nil
    ind = pat.data.index { |data| data.program_number != 0 }
    pid = pat.data[ind].pid if ind
    break unless pid
    pmt = get_pmt(fe, pid)
    filter = analyze_pmt(pmt)
    filter << [:other, PID::PAT]
    filter << [:other, pid]
    filter << [:other, PID::EIT]
    p filter

    File.open("/dev/null", "w") do |s|
      main(fe, s, filter)
    end
  end
end
