#include "ruby.h"

#include <stdio.h>

#include "pes.h"

VALUE rb_cPes, rb_ePesError;

static void free_pes(Pesdata *pes)
{
	if (pes) {
		free(pes);
	}
}

static VALUE pes_alloc(VALUE klass)
{
	return Data_Wrap_Struct(klass, 0, free_pes, 0);
}

static VALUE pes_hdr(VALUE self)
{
	Pesdata *pes;

	GetPes(self, pes);
	if (pes->hdr == 0)
		return Qfalse;
	else
		return Qtrue;
}

static VALUE pes_body(VALUE self)
{
	Pesdata *pes;

	GetPes(self, pes);
	if (pes->hdr == 0)
		return rb_str_new((const char *)pes->buf, TSPACKET_MAX);
	else
		return rb_str_new((const char *)pes->buf + 6, TSPACKET_MAX - 6);
}

static VALUE pes_inspect(VALUE self)
{
	Pesdata *pes;
	char str[128];

	GetPes(self, pes);
	if (pes->hdr == 0)
		sprintf(str, "<Pes bdy>");
	else
		sprintf(str, "<Pes hdr stream_id:0x%02x pid:0x%04x pes_packet_length:%d>",
				pes->stream_id, pes->pid, pes->pes_packet_length);
	return rb_str_new(str, strlen(str));
}

#define PES_DEFFUNC(xxxx)						\
	static VALUE pes_##xxxx(VALUE self)			\
{												\
	Pesdata *pes;								\
												\
	GetPes(self, pes);							\
	if (pes->hdr == 0)							\
		return Qnil;							\
	return INT2FIX(pes->xxxx);					\
}
PES_DEFFUNC(pid)
PES_DEFFUNC(packet_start_code_prefix)
PES_DEFFUNC(stream_id)
PES_DEFFUNC(pes_packet_length)

VALUE peshdr_create(unsigned char *packet, int len, unsigned int pid)
{
	VALUE self;
	Pesdata *pes;
	unsigned int packet_start_code_prefix;
	unsigned int pes_packet_length;

	if (packet == NULL || len < 6 || len > TSPACKET_MAX)
		return Qnil;
	packet_start_code_prefix = (packet[0] << 16) | (packet[1] << 8) | packet[2];
	if (packet_start_code_prefix != 0x00001)
		return Qnil;
	pes_packet_length = (packet[4] << 8) | packet[5];

	pes = ALLOC(Pesdata);
	if (pes == NULL)
		return Qnil;

	pes->hdr = 1;
	pes->pid = pid;
	pes->packet_start_code_prefix = packet_start_code_prefix;
	pes->stream_id = packet[3];
	pes->pes_packet_length = pes_packet_length;
	memcpy(pes->buf, packet, len);

	self = rb_funcall(rb_cPes, rb_intern("new"), 0);
	DATA_PTR(self) = pes;
	return self;
}

VALUE pesbdy_create(unsigned char *packet, int len, unsigned int pid)
{
	VALUE self;
	Pesdata *pes;

	if (packet == NULL || len < 6 || len > TSPACKET_MAX)
		return Qnil;

	pes = ALLOC(Pesdata);
	if (pes == NULL)
		return Qnil;

	pes->hdr = 0;
	memcpy(pes->buf, packet, len);

	self = rb_funcall(rb_cPes, rb_intern("new"), 0);
	DATA_PTR(self) = pes;
	return self;
}

void Init_pes(void)
{
    rb_cPes = rb_define_class("Pes", rb_cObject);
    rb_ePesError = rb_define_class("PesError", rb_eStandardError);

    rb_define_alloc_func(rb_cPes, pes_alloc);

	rb_define_method(rb_cPes, "packet_start_code_prefix", pes_packet_start_code_prefix, 0);
	rb_define_method(rb_cPes, "stream_id", pes_stream_id, 0);
	rb_define_method(rb_cPes, "pes_packet_length", pes_pes_packet_length, 0);
	rb_define_method(rb_cPes, "pid", pes_pid, 0);
	rb_define_method(rb_cPes, "hdr", pes_hdr, 0);
	rb_define_method(rb_cPes, "body", pes_body, 0);
	rb_define_method(rb_cPes, "inspect", pes_inspect, 0);
}

/*
 * Local Variables:
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * tab-width: 4
 * End:
 */
